---------Complex number Euphoria math library
constant inf = 1e+300 * 1e+20                 --(infinity)
constant nan = (inf / inf)                    --(not a number) 

--sgn : returns the sign of an argument
global function sgn(object r)                 --sign of r
  r = ((r > 0) - (r <= 0))                    -- real
  return r
end function
--abs : returns the absolute positive value of an argument
global function abs(object r)                 --absolute value of r
  r = sgn(r) * r                              -- real
  return (r != nan) * r
end function

---------- complex number operators -----------------------------------------
--c_mod : returns the modulus of a complex number
global function c_mod(sequence p)             --|x|          :sequence ok
  object x, y                                 -- complex arguments
  x = p[1] * p[1] y = p[2] * p[2]             -- real result
  return x + y 
end function
--capture : determines if a complex number was captured by a root
global function capture(sequence p, sequence q)--capture     :sequence ok
  return c_mod(p - q) < 0.00001               -- complex arguments 
end function                                  -- boolean result
--c_swap : exchanges the real and imaginary components of a complex number
global function c_swap(sequence p)            --swap         :sequence ok
  object x, y                                 -- complex arguments
  x = p[1] y = p[2]                           -- complex result
  if atom(x) then
    return {y, x}
  else
    return {y, x}
  end if
end function
--c_mul : multiplies two complex numbers
global function c_mul(sequence p, sequence q) --p * q        :sequence ok
  object a, b, u, v, x, y                     -- complex arguments
  u = p[1] v = p[2] x = q[1] y = q[2]         -- complex result
  a = u * x - v * y
  b = v * x + u * y
  return {a, b}
end function
--c_div : divides two complex numbers
global function c_div(sequence p, sequence q) --p / q        :sequence ok
  object a, b, u, v, x, y, determ             -- complex arguments
  determ = c_mod(q)                           -- complex result
  determ = (determ = 0) + determ
  u = p[1] v = p[2] x = q[1] y = q[2]
  a = (u * x + v * y) / determ
  b = (v * x - u * y) / determ
  return {a, b}
end function
--c_sqr : returns a complex number squared
global function c_sqr(sequence p)             --p^2          :sequence ok
  object a, b, x, y                           -- complex arguments 
  x = p[1] y = p[2]                           -- complex result
  a = x * x - y * y                           -- faster than c_mul(p, p)
  b = 2 * x * y
  return {a, b}
end function
--c_pwr : returns a complex number raised to a positive integer
global function c_pwr(sequence p, integer q)   --p^q          :sequence?
  sequence out                                -- complex base, integer power
  out = p                                     -- complex result
  while q > 1 do
    out = c_mul(out, p)
    q = q - 1
  end while
  return out
end function

